#!/usr/bin/python

import random

class Coin:
    def __init__(self, weight):
        self.weight = weight

class Weighs:
    def __init__(self, coins):
        self.coins   = coins
        self.left    = []
        self.right   = []
        self.counter = 3

    def put(self, side, coin):
        if coin < 0 or coin >= len(self.coins):
            raise Exception('Invalid coin number: ' + coin + ', must be between 0 and 11')

        if side == -1:
            self.left.append(self.coins[coin])
        elif side == 1:
            self.right.append(self.coins[coin])
        else:
            raise Exception('Invalid side: ' + side + ', expected -1 or 1')

    # -1: left is heavier than right
    #  0: balance
    #  1: left is lighter than right
    def weigh(self):
        if self.counter == 0:
            raise Exception('Too many weighs attempts')
        self.counter -= 1

        left = 0
        for coin in self.left:
            left += coin.weight
        right = 0
        for coin in self.right:
            right += coin.weight

        self.left  = []
        self.right = []

        if left < right: return  1
        if left > right: return -1
        return 0

def make_coins(fake, lighter):
    if fake is None:
        fake = int(random.random() * 12)
    elif fake < 0 or fake > 11:
        raise Exception('Invalid fake coin number: expected positive int less than 12, got ' + fake)

    coins = [ 1 for i in range(12) ]
    coins[fake] = 0 if lighter else 2
    return [ Coin(w) for w in coins ]

def test(solver):
    solved = True
    for fake in range(0, 12):
        for lighter in [ True, False ]:
            f, l = solve(Weighs(make_coins(fake, lighter)))
            success = f == fake and l == lighter
            solved = solved and success
            print(
                    fake,
                    'light' if lighter else 'heavy',
                    'success' if success else 'fail: coin ' + str(f) + ', weight ' + str(l)
            )
    print('Problem solved!' if solved else 'Fail')
    return solved

def solve(w):
    for i in range(4):
        w.put(-1, i)
    for i in range(4, 8):
        w.put( 1, i)
    w1 = w.weigh()
    if w1 == 0:
        w.put(-1, 8)
        w.put(-1, 9)
        w.put( 1, 0)
        w.put( 1, 1)
        w2 = w.weigh()
        if w2 == 0:
            w.put(-1, 10)
            w.put( 1, 0)
            w3 = w.weigh()
            if w3 == 0:
                return 11, None # lighter? heavier?
            return 10, w3 > 0
        w.put(-1, 8)
        w.put( 1, 9)
        w3 = w.weigh()
        if w3 == w2:
            return 8, w3 > 0
        return 9, w3 < 0
    w.put(-1, 0)
    w.put(-1, 1)
    w.put(-1, 4)
    w.put(-1, 5)
    for i in range(8, 12):
        w.put(1, i)
    w2 = w.weigh()
    if w2 == w1:
        w.put(-1, 8)
        w.put( 1, 9)
        w3 = w.weigh()
        if w3 == w2:
            return 8, w2 > 0
        return 9, w2 > 0
    elif w2 == -w1:
        w.put(-1, 0)
        w.put( 1, 1)
        w3 = w.weigh()
        if w3 == w2:
            return 1, w2 < 0
        return 0, w2 < 0
    return None, None


test(solve)
